@@ -193,6 +193,8 @@ class InitTestModelSubclass(InitTestModel):
 # models from github issue
 class Top(PolymorphicModel):
     name = models.CharField(max_length=50)
+    class Meta:
+        ordering = ('pk',)
 class Middle(Top):
     description = models.TextField()
 class Bottom(Middle):
@@ -221,6 +223,9 @@ class ProxyChild(ProxyBase):
     class Meta:
         proxy = True
 
+class NonProxyChild(ProxyBase):
+    name=models.CharField(max_length=10)
+
 # base -> proxy -> real models
 class ProxiedBase(ShowFieldTypeAndContent, PolymorphicModel):
     name = models.CharField(max_length=10)
@@ -714,6 +719,24 @@ class PolymorphicTests(TestCase):
 
         self.assertIsInstance(items[0], ProxyChild)
 
+    def test_proxy_get_real_instance_class(self):
+        """
+        The call to ``get_real_instance()`` also checks whether the returned model is of the correct type.
+        This unit test guards that this check is working properly. For instance,
+        proxy child models need to be handled separately.
+        """
+        name="Item1"
+        nonproxychild = NonProxyChild.objects.create(name=name)
+
+        pb = ProxyBase.objects.get(id=1)
+        self.assertEqual(pb.get_real_instance_class(), NonProxyChild)
+        self.assertEqual(pb.get_real_instance(), nonproxychild)
+        self.assertEqual(pb.name, name)
+
+        pbm = ProxyChild.objects.get(id=1)
+        self.assertEqual(pbm.get_real_instance_class(), NonProxyChild)
+        self.assertEqual(pbm.get_real_instance(), nonproxychild)
+        self.assertEqual(pbm.name, name)
 
     def test_content_types_for_proxy_models(self):
         """Checks if ContentType is capable of returning proxy models."""
